@@ -35,6 +35,8 @@ Config:
                 Channel title.
     strip-name-emoji (bool):
         ``True`` to remove emoji characters from message authors' real names.
+    titles ((str, str) dict):
+        Mapping from virtual channel names to display names.
 
 Commands:
     sync-members:
@@ -100,7 +102,10 @@ from itertools import chain
 import logging
 import re
 
-from peewee import CharField
+try:
+    from peewee import CharField
+except ImportError:
+    CharField = None
 
 import immp
 from immp.hook.command import command
@@ -131,78 +136,80 @@ def _emoji_replace(match):
     return " " if left and right else ""
 
 
-class SyncBackRef(BaseModel):
-    """
-    One of a set of references, each pointing to a representation of a source message.
+if CharField:
 
-    Attributes:
-        key (str):
-            Shared ID for each message in the same set.
-        network (str):
-            Network identifier of the plug for this message.
-        channel (str):
-            Origin channel of the referenced message.
-        message (str):
-            Message ID as generated by the plug.
-    """
-
-    key = CharField()
-    network = CharField()
-    channel = CharField()
-    message = CharField()
-
-    class Meta:
-        # Unique constraint for each message.
-        indexes = ((("network", "channel", "message"), True),)
-
-    @classmethod
-    def map_from_sent(cls, sent):
+    class SyncBackRef(BaseModel):
         """
-        Take a :class:`.Receipt` and attempt to resolve it to a key from a previously synced
-        reference, then defer to :meth:`map_from_key` to make the mapping with the key.
+        One of a set of references, each pointing to a representation of a source message.
 
-        Args:
-            sent (.Receipt):
-                Referenced message to lookup.
-
-        Returns:
-            ((str, str), .SyncBackRef) dict:
-                Generated reference mapping.
-        """
-        try:
-            key = cls.select(cls.key).where(cls.network == sent.channel.plug.network_id,
-                                            cls.channel == sent.channel.source,
-                                            cls.message == str(sent.id)).get().key
-        except cls.DoesNotExist:
-            raise KeyError
-        else:
-            return cls.map_from_key(key)
-
-    @classmethod
-    def map_from_key(cls, key):
-        """
-        For a given key, fetch all messages references synced from the corresponding source, and
-        group them by network and channel.
-
-        Args:
+        Attributes:
             key (str):
-                Synced message identifier.
-
-        Returns:
-            ((str, str), .SyncBackRef) dict:
-                Generated reference mapping.
+                Shared ID for each message in the same set.
+            network (str):
+                Network identifier of the plug for this message.
+            channel (str):
+                Origin channel of the referenced message.
+            message (str):
+                Message ID as generated by the plug.
         """
-        backrefs = cls.select().where(cls.key == key)
-        if not backrefs:
-            raise KeyError
-        mapped = defaultdict(list)
-        for backref in backrefs:
-            mapped[(backref.network, backref.channel)].append(backref)
-        return key, mapped
 
-    def __repr__(self):
-        return "<{}: #{} {} {} @ {}/{}>".format(self.__class__.__name__, self.id, self.key,
-                                                self.message, self.network, self.channel)
+        key = CharField()
+        network = CharField()
+        channel = CharField()
+        message = CharField()
+
+        class Meta:
+            # Unique constraint for each message.
+            indexes = ((("network", "channel", "message"), True),)
+
+        @classmethod
+        def map_from_sent(cls, sent):
+            """
+            Take a :class:`.Receipt` and attempt to resolve it to a key from a previously synced
+            reference, then defer to :meth:`map_from_key` to make the mapping with the key.
+
+            Args:
+                sent (.Receipt):
+                    Referenced message to lookup.
+
+            Returns:
+                ((str, str), .SyncBackRef) dict:
+                    Generated reference mapping.
+            """
+            try:
+                key = cls.select(cls.key).where(cls.network == sent.channel.plug.network_id,
+                                                cls.channel == sent.channel.source,
+                                                cls.message == str(sent.id)).get().key
+            except cls.DoesNotExist:
+                raise KeyError
+            else:
+                return cls.map_from_key(key)
+
+        @classmethod
+        def map_from_key(cls, key):
+            """
+            For a given key, fetch all messages references synced from the corresponding source, and
+            group them by network and channel.
+
+            Args:
+                key (str):
+                    Synced message identifier.
+
+            Returns:
+                ((str, str), .SyncBackRef) dict:
+                    Generated reference mapping.
+            """
+            backrefs = cls.select().where(cls.key == key)
+            if not backrefs:
+                raise KeyError
+            mapped = defaultdict(list)
+            for backref in backrefs:
+                mapped[(backref.network, backref.channel)].append(backref)
+            return key, mapped
+
+        def __repr__(self):
+            return "<{}: #{} {} {} @ {}/{}>".format(self.__class__.__name__, self.id, self.key,
+                                                    self.message, self.network, self.channel)
 
 
 class SyncRef:
@@ -431,7 +438,7 @@ class SyncPlug(immp.Plug):
         return False if channel.source in self._hook.config["channels"] else None
 
     async def channel_title(self, channel):
-        return channel.source
+        return self._hook.config["titles"].get(channel.source, channel.source)
 
     async def channel_members(self, channel):
         if channel.source not in self._hook.config["channels"]:
@@ -596,7 +603,8 @@ class SyncHook(_SyncHookBase):
 
     schema = immp.Schema({immp.Optional("joins", True): bool,
                           immp.Optional("renames", True): bool,
-                          immp.Optional("plug"): immp.Nullable(str)}, _SyncHookBase.schema)
+                          immp.Optional("plug"): immp.Nullable(str),
+                          immp.Optional("titles", dict): {str: str}}, _SyncHookBase.schema)
 
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
